#!/bin/sh
#
# Copyright (c) 2022  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

skip_all()
{
	local first="$1" last="$2" reason="$3"
	local idx="$first"
	while [ "$idx" -le "$last" ]; do
		echo "ok $idx # skip $reason"
		idx=$((idx + 1))
	done
}

[ -z "$CONFGET" ] && CONFGET='./confget'
[ -z "$TESTDIR" ] && TESTDIR='t'

T5="$TESTDIR/t5.ini"

echo '1..9'

if [ ! -f "$T5" ]; then
	echo "Bail out!  No test file $T5"
	exit 255
fi

if ! $CONFGET -qfeatures | tr ' ' '\n' | grep -Eqe '^REGEX='; then
	skip_all 1 9 'No regular expression support in confget'
	exit 0
fi

singloc="$(locale -a | grep -Eie '\.iso-?8859-?(1|15)([^0-9]|$)' | head -n1)"
if [ -z "$singloc" ]; then
	skip_all 1 9 'No ISO-8859-1 or ISO-8859-15 locale configured'
	exit 0
fi
echo "# Using '$singloc' as a single-byte locale name"

u8loc="$(locale -a | grep -Eie '\.utf-?8([^0-9]|$)' | head -n1)"
if [ -z "$u8loc" ]; then
	skip_all 1 9 'No UTF-8 locale configured'
	exit 0
fi
echo "# Using '$u8loc' as a multibyte locale name"

if $CONFGET -f "$T5" -s 'latin' -x -l -N -m '^.' | grep -Eqe '^enye='; then echo 'ok 1'; else echo "not ok 1 rc $?"; fi

# In a non-UTF-8 environment, that letter should be treated as two characters.
if env "LC_ALL=$singloc" $CONFGET -f "$T5" -s 'latin' -x -l -N -m '^..$' | grep -Eqe '^enye='; then echo 'ok 2'; else echo "not ok 2 rc $?"; fi
if ! env "LC_ALL=$singloc" $CONFGET -f "$T5" -s 'latin' -x -l -N -m '^.$' | grep -Eqe '^enye='; then echo 'ok 3'; else echo "not ok 3 rc $?"; fi

if env "LC_ALL=$singloc" $CONFGET -f "$T5" -s 'latin' -l -N -m '??' | grep -Eqe '^enye='; then echo 'ok 4'; else echo "not ok 4 rc $?"; fi
if ! env "LC_ALL=$singloc" $CONFGET -f "$T5" -s 'latin' -l -N -m '?' | grep -Eqe '^enye='; then echo 'ok 5'; else echo "not ok 5 rc $?"; fi

# In a UTF-8 environment, that letter should be treated as a single character.
if ! env "LC_ALL=$u8loc" $CONFGET -f "$T5" -s 'latin' -x -l -N -m '^..$' | grep -Eqe '^enye='; then echo 'ok 6'; else echo "not ok 6 rc $?"; fi
if env "LC_ALL=$u8loc" $CONFGET -f "$T5" -s 'latin' -x -l -N -m '^.$' | grep -Eqe '^enye='; then echo 'ok 7'; else echo "not ok 7 rc $?"; fi

if $CONFGET -q features | grep -Fqe ' REGEX_IMPL_C_'; then
	# Skipped because of a GNU libc fnmatch() bug, see https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=1024974
	echo 'ok 8 # skip GNU libc fnmatch bug'
else
	if ! env "LC_ALL=$u8loc" $CONFGET -f "$T5" -s 'latin' -l -N -m '??' | grep -Eqe '^enye='; then echo 'ok 8'; else echo "not ok 8 rc $?"; fi
fi
if env "LC_ALL=$u8loc" $CONFGET -f "$T5" -s 'latin' -l -N -m '?' | grep -Eqe '^enye='; then echo 'ok 9'; else echo "not ok 9 rc $?"; fi
